/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* This file is based on the "small client-server example" from Qt */
/* Copyright (C) 1992-2000 Trolltech AS. */


#include "server.h"
#include <qtextstream.h>
#include <qregexp.h>
#include "version.h"

ClientSocket::ClientSocket( int sock, QObject *parent, const char *name ) 
	: QSocket( parent, name )
{
	connect( this, SIGNAL(readyRead()), SLOT(readClient()) );
	connect( this, SIGNAL(connectionClosed()), SLOT(deleteLater()) );
	setSocket( sock );

	sendText("SMPlayer " VERSION);
	sendText("Type help for a list of commands");
}

ClientSocket::~ClientSocket() {
}

void ClientSocket::readClient() {
	QTextStream ts( this );
	while ( canReadLine() ) {
		QString str = ts.readLine();
		qDebug("ClientSocket::readClient: '%s'", str.utf8().data() );

		if (str.lower() == "hello") {
			sendText("Hello, this is SMPlayer " VERSION);
		}
		else
		/*
		if (str.startsWith("Hello, this is smplayer")) {
			sendText("I'm smplayer too, go on");
		} 
		else
		*/
		if (str.lower() == "help") {
			sendText("Available commands:");
			sendText(" help");
			sendText(" quit");
			sendText(" open [file]");
		}
		else
		if (str.lower() == "quit") {
			sendText("Goodbye");
			flush();
			close();
		}
		else {
			QRegExp rx("^open (.*)");
			if (rx.search(str) > -1) {
				QString file = rx.cap(1);
				qDebug(" * asked to open '%s'", file.utf8().data());
				emit receivedOpen(file);
			} 
		}
	}
}

void ClientSocket::sendText(QString text) {
	QTextStream os(this);
	os << text << "\n";
}

Server::Server( Q_UINT16 port, QObject* parent ) 
	: QServerSocket( port, 1, parent )
{
	if ( !ok() ) {
		qWarning("Server::Server: failed to bind to port %d", port);
	} else {
		qDebug("Server::Server:: server started at port %d", port);
	}
}

Server::~Server() {
}

void Server::newConnection( int socket ) {
	ClientSocket *s = new ClientSocket( socket, this );
	connect(s, SIGNAL(receivedOpen(QString)), 
            this, SLOT(gotOpenCommand(QString)));
	emit newConnect( s );
}

void Server::gotOpenCommand(QString text) {
	emit receivedOpen(text);
}

